/*
 * Copyright (c) 2015, Georgia Tech Research Corporation
 * All rights reserved.
 *
 * Author(s): Michael X. Grey <mxgrey@gatech.edu>
 *
 * Georgia Tech Graphics Lab and Humanoid Robotics Lab
 *
 * Directed by Prof. C. Karen Liu and Prof. Mike Stilman
 * <karenliu@cc.gatech.edu> <mstilman@cc.gatech.edu>
 *
 * This file is provided under the following "BSD-style" License:
 *   Redistribution and use in source and binary forms, with or
 *   without modification, are permitted provided that the following
 *   conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *   CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *   POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef KIDO_GUI_OSG_MOUSEEVENTHANDLER_HPP
#define KIDO_GUI_OSG_MOUSEEVENTHANDLER_HPP

#include "kido/common/Subject.hpp"
#include "kido/common/Observer.hpp"

#include "kido/gui/osg/DefaultEventHandler.hpp"

namespace kido {
namespace gui {
namespace osg {

class DefaultEventHandler;

class MouseEventHandler : public virtual kido::common::Subject,
                          public virtual kido::common::Observer
{
public:

  friend class DefaultEventHandler;

  inline MouseEventHandler() : mEventHandler(nullptr) { }

  /// Overload this function to set behavior that will get triggered during a
  /// mouse event
  virtual void update() = 0;

protected:

  inline virtual void handleDestructionNotification(const Subject* _subject) override
  {
    if(mEventHandler == _subject)
      delete this;
  }

  DefaultEventHandler* mEventHandler;

};

} // namespace osg
} // namespace gui
} // namespace kido

#endif // KIDO_GUI_OSG_MOUSEEVENTHANDLER_HPP
